@@ -17,7 +17,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * Contact roland@cpan.org for any licensing questions.
+ * A commercial license is available as well. You can either apply the GPL or
+ * obtain a commercial license for closed source development. Please contact
+ * roland@cpan.org for further information.
  */
 
 package org.cpan.jmx4perl;
@@ -61,6 +63,9 @@ import java.util.Map;
  * are supported as well. Refer to {@link org.cpan.jmx4perl.converter.AttributeToJsonConverter}
  * for additional information.
  *
+ * For the client part, please read the documentation of
+ * <a href="http://search.cpan.org/dist/jmx4perl">jmx4perl</a>.
+ *
  * @author roland@cpan.org
  * @since Apr 18, 2009
  */
@@ -112,9 +117,15 @@ public class AgentServlet extends HttpServlet {
                 throw new UnsupportedOperationException("Unsupported operation '" + jmxReq.getType() + "'");
             }
             json = jsonConverter.convertToJson(retValue,jmxReq);
-            json.put("status",200 /* success */);  
+            json.put("status",200 /* success */);
+        } catch (AttributeNotFoundException exp) {
+            code = 404;
+            throwable = exp;
+        } catch (InstanceNotFoundException exp) {
+            code = 404;
+            throwable = exp;
         } catch (UnsupportedOperationException exp) {
-            code = 501;
+            code = 404;
             throwable = exp;
         } catch (IllegalArgumentException exp) {
             code = 400;
@@ -146,7 +157,7 @@ public class AgentServlet extends HttpServlet {
         return jsonObject;
     }
 
-    private Object listMBeans() {
+    private Object listMBeans() throws InstanceNotFoundException {
         try {
             MBeanServer server = getMBeanServer();
             Map<String /* domain */,
@@ -173,8 +184,6 @@ public class AgentServlet extends HttpServlet {
                 }
             }
             return ret;
-        } catch (InstanceNotFoundException e) {
-            throw new IllegalStateException("Internal error while retrieving list: " + e,e);
         } catch (ReflectionException e) {
             throw new IllegalStateException("Internal error while retrieving list: " + e,e);
         } catch (IntrospectionException e) {
@@ -228,14 +237,10 @@ public class AgentServlet extends HttpServlet {
         writer.write(pJsonTxt);
     }
 
-    private Object getMBeanAttribute(JmxRequest pJmxReq) {
+    private Object getMBeanAttribute(JmxRequest pJmxReq) throws AttributeNotFoundException, InstanceNotFoundException {
         try {
             wokaroundJBossBug(pJmxReq);
             return getMBeanServer().getAttribute(pJmxReq.getObjectName(), pJmxReq.getAttributeName());
-        } catch (AttributeNotFoundException ex) {
-            throw new IllegalArgumentException("No attribute '" + pJmxReq.getAttributeName() + "' :" + ex);
-        } catch (InstanceNotFoundException ex) {
-            throw new IllegalArgumentException("No object with name '" + pJmxReq.getObjectName() + "' : " + ex);
         } catch (ReflectionException e) {
             throw new RuntimeException("Internal error for " + pJmxReq.getAttributeName() +
                     "' on object " + pJmxReq.getObjectName() + ": " + e);
@@ -317,7 +322,6 @@ public class AgentServlet extends HttpServlet {
         return nMap;
     }
 
-
     private boolean checkForClass(String pClassName) {
         try {
             Class.forName(pClassName);
